/* Part 1 - separated into 2 parts to overcome 32K segment limitation
            of Macintosh */
            
/************************************************************************
*      H2O property functions based on:
*          L. Harr, J. Gallagher and G. Kell, "NBS/NRC Steam Tables,
*          Thermodynamic and Transport Properties and Computer Programs
*          for Vapor and Liquid States of Water in SI Units",
*          Hemisphere Publishing Corp., McGraw-Hill International Book
*          Company, 1984, ISBN 0-89116-354-9.
*
*      These functions are low-order fits that are fast running and
*      reasonably accurate (usually < 0.1% error). For more information
*      refer to the following publications:
*
*
*      Please feel free to distribute copies of these functions since the
*      general idea is to promote the free exchange of useful code.
*      Please DO NOT distribute altered copies; the world is confusing
*      enough as it is. I would appreciate being informed of any errors,
*      ommisions, comments, etc.
*
*          Wm. J. Garland
*          Associate Professor
*          Department of Engineering Physics
*          McMaster University
*          1280 Main St. W.
*          Hamilton, Ontario
*          Canada    L8S 4M1
*          Ph: (416) 525-9140 ext. 4925
*
*************************************************************************
*
* Saturation properties based on reference 1 (Garland and Hoskins)
*
* Subcooled and superheated properties based on reference 2 (Garland and
* Hand)
*         SAT means saturated
*         SUB means subcooled
*         SUP means superheated
*
***********************************************************************/
#include <math.h>
#include "h2o.h"
#define  ABS(x) ((x) >= 0 ? (x) : -(x))

float CPF (float P)
/* This function calculates the specific heat of light water
   for a given pressure.*/
{
      extern char *ERRH2O;
      float CPF;
      if (P < 0.03)   {
           CPF=0.247763* pow(0.03,0.5704026) +4.150;
           ERRH2O=" CPF: PRESSURE LESS THAN 0.03 MPa ";
      }
      else if (P > 20.3)   {
           CPF= 4.6844786E-6*exp(0.7396875*P)+10.020;
           ERRH2O=" CPF: PRESSURE GREATER THAN 20.3 MPa ";
      }
      else if (P >= 0.030 && P <= 0.671)   {
           CPF=0.247763* pow(P,0.5704026) +4.150;
      }
      else if (P > 0.671 && P <= 2.606)   {
         CPF=0.1795305* pow(P,0.8967323) +4.223;
      }
      else if (P > 2.606 && P <= 6.489)   {
           CPF=0.09359843* pow(P,1.239114) +4.340;
      }
      else if (P > 6.489 && P <= 11.009)   {
           CPF=0.01068888* pow(P,2.11376) +4.740;
      }
      else if (P > 11.009 && P <= 14.946)   {
           CPF=1.333058E-4* pow(P,3.707294) +5.480;
      }
      else if  (P > 14.946 && P <= 18.079)   {
           CPF=6.635658E-3* pow((P -10.0),3.223323)+7.350;
      }
      else
           CPF= 4.6844786E-6*exp(0.7396875*P)+10.020;
      return (CPF);
}

/*************************************************************************/
float CPFSUB (float P, float T)
/* This function calculates the heat capacity of light water
   in the subcooled liquid phase.*/
{
      extern char *ERRH2O;
      float PS,PP,TT;
      float PSAT(float T);
      float CPF(float P);
      PS=PSAT(T);
      PP=P;
      TT=T;
      if (P < PS)   {
             PP=PS;
          ERRH2O=" CPFSUB: PRESSURE LESS THAN SATURATION ";
      }
      else if (P > 20.3)   {
          PP=20.3;
          ERRH2O=" CPFSUB: PRESSURE GREATER THAN 20.3 MPa ";
      }
      else if (T < 89.96)   {
          TT=89.96;
          ERRH2O=" CPFSUB: TEMPERATURE LESS THAN 89.96 deg C ";
      }
      else if (T > 357.03)   {
         TT=357.03;
          ERRH2O=" CPFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
      }
      return (CPF(PS) + (0.0018-76./pow((364.-TT),1.8))*(PP-PS));
}

/************************************************************************/
float CPG(float P)
/* This function calculates the specific heat of light water
   in the vapour phase for a given pressure.*/
{
      extern char *ERRH2O;
      float CPG;
      if (P < 0.05)   {
             CPG=0.6471635* pow((0.05-0.006),0.6400569) +1.90;
             ERRH2O=" CPG: PRESSURE LESS THAN 0.05 MPa";
      }
      else if   (P > 20.4)   {
             CPG= 6.5162612E-6*exp(0.7562118*20.4)+17.10;
             ERRH2O=" CPG: PRESSURE GREATER THAN 20.4 MPa ";
      }
      else if   (P >= 0.050 && P <= 0.599)   {
             CPG=0.6471635* pow((P-0.006),0.6400569) +1.90;
      }
      else if   (P > 0.599 && P <= 2.391)   {
             CPG=0.5560633* pow(P,0.8197355) +2.00;
      }
      else if   (P > 2.391 && P <= 5.661)   {
             CPG=0.3187082* pow(P,1.110271) +2.3;
      }
      else if   (P > 5.661 && P <= 9.458)   {
             CPG=0.064275995* pow(P,1.766106) +3.12;
      }
      else if   (P > 9.458 && P <= 12.900)   {
             CPG=3.8011048E-3* pow(P,2.816897) +4.40;
      }
      else if   (P > 12.900 && P <= 16.309)   {
             CPG=0.1876175* exp(0.2466925*P)+5.00;
      }
      else if   (P > 16.309 && P <= 18.743)   {
             CPG=7.62075E-3* exp(P*0.4117289)+9.20;
      }
      else   {
             CPG= 6.5162612E-6*exp(0.7562118*P)+17.10;
      }
      return(CPG);
}

/************************************************************************/
float CPGSUP(float P,float T)
/* This function calculates the heat capacity of light water
   in the superheated gaseous phase.*/
{
      extern char *ERRH2O;
      float TS,PP,TT;
      float TSAT(float P);
      float CPG(float P);
      float res;
      TS=TSAT(P);
      PP=P;
      TT=T;
      if (P < 0.07)   {
         PP=0.07;
          ERRH2O=" CPGSUP: PRESSURE LESS THAN 0.07 MPa ";
      }
      else if (P > 14.0)   {
         PP=14.;
          ERRH2O=" CPGSUP: PRESSURE GREATER THAN 14 MPa ";
      }
      else if (T < TS)   {
         TT=TS;
          ERRH2O=" CPGSUP: TEMPERATURE LESS THAN SATURATION ";
      }
      else if   (T > 400.0)   {
         TT=400.;
          ERRH2O=" CPGSUP: TEMPERATURE GREATER THAN 400 deg C        ";
      }
      res=CPG(P) -(0.011*PP/pow(0.00014*(TT+8.0)*(TT+8.0)-PP,1.5)+1.5E-8 *pow((655.0-TT),2.1))*PP*(TT-TS);
      return(res);
}

/**********************************************************************/
float DF(float P)
/* This function calculates the density of light water
   in liquid phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float DF;
      if (P < 0.002)     {
          DF=1./(1.9118E-4* pow(0.002,0.546472) + 0.0009947);
          ERRH2O=" DF: PRESSURE LESS THAN 0.002 MPa    ";
      }
      else if (P > 21.5)     {
          DF=1./(4.1035988E-24* pow(21.5,15.03329) +0.00189);
          ERRH2O=" DF: PRESSURE GREATER THAN 21.5 MPa       ";
      }
      else if (P >= 0.002  &&  P < 0.01468)     {
          DF=1./ (1.9118E-4 * pow(P,0.546472) + 0.0009947);
      }
      else if (P >= 0.01468  &&  P < 0.275)     {
          DF=1./ (1.380934E-4 * pow(P,0.388715) + 0.000987);
      }
      else if (P >= 0.275 && P <= 1.0)     {
          DF=1./(1.2746977E-4* pow(P,0.4644339) +0.001);
      }
      else if (P > 1.0 && P <= 3.88)     {
          DF=1./(1.0476071E-4* pow(P,0.5651090) +0.001022);
      }
      else if (P > 3.88 && P <= 8.84)     {
          DF=1./(3.2836717E-5* P +1.12174735E-3);
      }
      else if (P > 8.84 && P <= 14.463)     {
          DF=1./(3.3551046E-4 * exp(0.058403566 *P) +0.00085);
      }
      else if (P > 14.463 && P < 18.052)     {
          DF= 1./(3.1014626E-8* pow(P,3.284754) +0.00143);
      }
      else if (P >= 18.052 && P < 20.204)     {
          DF= 1./(1.5490787E-11* pow(P,5.7205) +0.001605);
      }
      else     {
          DF= 1./(4.1035988E-24* pow(P,15.03329) +0.00189);
      }
      return(DF);
}   /* end of function DF(P) */

/**********************************************************************/
float DFSUB(float P,float T)
/* This function calculates the density of light water
   in the subcooled liquid phase.*/
{
      extern char *ERRH2O;
      float PS,PP,TT,DFSUB;
      float PSAT(float P);
      float DF(float P);
      PS=PSAT(T);
      PP=P;
      TT=T;
      if (P < PS)     {
        PP=PS;
        ERRH2O=" DFSUB: PRESSURE LESS THAN SATURATION ";
      }
      else if (P > 18.0)     {
        PP=18.0;
        ERRH2O=" DFSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < 17.511)     {
        TT=17.511;
        ERRH2O=" DFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
      }
      else if (T > 357.03)     {
        TT=357.03;
        ERRH2O=" DFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
      }
      DFSUB = DF(PS) + (170.0 / (375.0-TT) - 0.2) * ( PP-PS );
      return(DFSUB);
}   /* end of function DFSUB(P,T) */

/**********************************************************************/
float DG(float P)
/* This function calculates the density of light water
   in the gas phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float DG;
      if (P < 0.002)     {
             DG= 5.0981616 * pow(0.002,0.936226) - 0.00025;
             ERRH2O=" DG: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 21.5)     {
             DG= 3.558734E-16* pow(21.5,13.03774) +138.0;
             ERRH2O=" DG: PRESSURE GREATER THAN 21.5 MPa ";
      }
      else if (P >= 0.002  &&  P < 0.2139)     {
             DG= 5.0981616 * pow(P,0.936226) - 0.00025;
      }
      else if (P >= 0.2139 && P < 1.112)     {
             DG= 5.126076* pow(P,0.9475862) +0.012;
      }
      else if (P >= 1.112 && P < 3.932)     {
             DG= 4.630832* pow(P,1.038819) +0.52;
      }
      else if (P >= 3.932 && P < 8.996)     {
             DG= 2.868721* pow(P,1.252148) +3.80;
      }
      else if (P >= 8.996 && P < 14.628)
      {
             DG= 0.5497653* pow(P,1.831182) +18.111;
      }
      else if (P >= 14.628 && P <= 18.21)     {
             DG= 8.5791582E-3* pow(P,3.176484) +50.0;
      }
      else if (P > 18.21 && P <= 20.253)     {
            DG= 3.5587113E-6* pow(P,5.660939) +88.0;
      }
      else     {
            DG= 3.558734E-16* pow(P,13.03774) +138.0;
      }
      return(DG);
}   /* end of function DG(P) */

/**********************************************************************/
float DGSUP(float P,float T)
/* This function calculates the density of light water
   in the superheated gaseous phase.*/
{
      extern char *ERRH2O;
      float TS,PP,TT,DGSUP;
      float TSAT(float P);
      float VG(float P);
      TS=TSAT(P);
      PP=P;
      TT=T;
      if (P < 0.002)     {
             PP=0.002;
             ERRH2O=" DGSUP: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 18.0)     {
             PP=18.0;
             ERRH2O=" DGSUP: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < TS)     {
             TT=TS;
             ERRH2O=" DGSUP: TEMPERATURE LESS THAN SATURATION ";
      }
      else if (T > 450.0)     {
             TT=450.0;
             ERRH2O=" DGSUP: TEMPERATURE GREATER THAN 450 deg C      ";
      }
           DGSUP = pow(VG(PP)+(0.000466/PP-(0.12/(TT+100.0)-0.00106)
                    *pow(PP,0.1)/sqrt(1.96E-08*pow(TT+8.,4.0)-PP*PP))*(TT-TS),-1.0);
      return(DGSUP);
}   /* end of function DGSUP(P,T) */

/**********************************************************************/
float DHFDP(float P)
/* This function calculates the derivative with respect to
   pressure of the specific enthalpy of light water in the
   liquid phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float DHFDP;
      if (P < 0.002)     {
             DHFDP= 1128.777*0.135196* pow(0.002,0.135196 -1.0);
             ERRH2O=" DHFDP: PRESSURE LESS THAN 0.002 MPa    ";
      }
       else if (P > 21.7)     {
             DHFDP= 9.1417257E-14* 11.47287* pow(21.7,10.47287);
             ERRH2O=" DHFDP: PRESSURE GREATER THAN 21.7 MPa       ";
      }
      else if(P >= 0.002  &&  P < 0.0173)     {
             DHFDP=1128.777 * 0.135196 * pow(P,(0.135196-1.0));
      }
      else if(P >= 0.0173  &&  P < 0.1028)     {
             DHFDP=1050.7085 * 0.161797 * pow(P,(0.161797-1.0));
      }
      else if (P >= 0.1028 && P < 0.942)     {
             DHFDP= 912.1779*0.2061637* pow(P,(0.2061637 -1.0));
      }
      else if (P >= 0.942 && P < 4.02)     {
             DHFDP= 638.0621*0.2963192* pow(P,(0.2963192 -1.0));
      }
      else if (P >= 4.02 && P < 9.964)     {
             DHFDP= 373.7665*0.4235532* pow(P,(0.4235532 -1.0));
      }
      else if (P >= 9.964 && P < 16.673)     {
             DHFDP= 75.38673*0.8282384* pow(P,(0.8282384 -1.0));
      }
      else if (P >= 16.673 && P < 20.396)     {
             DHFDP= 0.1150827* 2.711412* pow(P,1.711412);
      }
      else    {
             DHFDP= 9.1417257E-14* 11.47287* pow(P,10.47287);
      }
      return(DHFDP);
}     /* end of function DHFDP (P) */

/**********************************************************************/
float DHFDPT(float P,float T)
/* This function calculates dh/dP |T for light water
   in the subcooled liquid phase.*/
{
      float DHFDPT;
      DHFDPT = 1.4-169.0/(369.0-T);
      return(DHFDPT);
}   /* end of function DHFDPT(P,T) */

/**********************************************************************/
float DHFDTP(float P,float T)
/* This function calculates dh/dT |P for light water
   in the subcooled liquid phase.*/
{
      float PS,DHFDTP;
      float PSAT(float P);
      float DPDT(float P);
      float DHFDP(float PS);
      PS=PSAT(T);
      DHFDTP = -169.0/((369.0-T)*(369.0-T))*(P-PS)+DPDT(T)
            * (DHFDP(PS)-1.4+169.0/(369.0-T));
      return(DHFDTP);
}   /* end of function DHFDTP(P,T) */

/**********************************************************************/
float DHGDP(float P)
/* This function calculates the derivative with respect to
   pressure of the specific enthalpy of light water in the gas
   phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float DHGDP;
      if (P < 0.002)     {
             DHGDP= 529.44008 * 0.108652 * pow(0.002,0.108652-1.0);
             ERRH2O=" DHGDP: PRESSURE LESS THAN 0.002 MPa    ";
      }
      else if (P > 21.55)     {
             DHGDP= -22.92521*2.0* (21.55-18.0) +44.23671;
             ERRH2O=" DHGDP: PRESSURE GREATER THAN 21.55 MPa       ";
      }
      else if(P >= 0.002  &&  P < 0.1379)     {
             DHGDP= 529.44008 * 0.108652 * pow(P,(0.108652-1.0));
      }
      else if (P >= 0.1379 && P <= 0.348)     {
             DHGDP= 4.0381938E-6*15.72364* pow(3.0-P,14.72364);
      }
      else if (P > 0.348 && P <= 1.248)     {
             DHGDP= 0.5767304*1.66153* exp(-1.66153* (P-3.2));
      }
      else if (P > 1.248 && P < 2.955)     {
             DHGDP= 7.835986*2.0* (3.001-P) -2.934312;
      }
      else if (P >= 2.955 && P <= 6.522)     {
             DHGDP= -1.347244*2.0* (P-2.999) -2.326913;
      }
      else if (P > 6.522 && P < 16.497)     {
             DHGDP= -0.9219176*2.0* (P-9.0) -16.38835;
      }
      else if (P >= 16.497 && P < 20.193)     {
             DHGDP= -3.432177*2.0* (P-8.0) +29.81305;
      }
      else    {
             DHGDP= -22.92521*2.0* (P-18.0) +44.23671;
      }
      return(DHGDP);
}   /* end of function DHGDP(P) */

/**********************************************************************/
float DHGDPT(float P,float T)
/* This function calculates dh/dP |T for light water
   in the superheated gaseous phase.*/
{
      float TS,PS,PTEMP,DHGDPT;
      float TSAT(float P);
      float PSAT(float T);
      float DTDP(float P);
      TS=TSAT(P);
      PS=PSAT(T);
      PTEMP=P;
      P = P + 0.13 * (PS-P);
      DHGDPT=DHGDP(P)-(4.5*P/sqrt(7.4529E-06*T*T*T-P*P)
              +0.28*exp(-0.008*(T-162.0))-100.0/T+2.225)*DTDP(P)+(T-TS)
              * (4.5/sqrt(7.4529E-06*T*T*T-P*P)
              +(4.5*P*P/pow(7.4529E-06*T*T*T-P*P,1.5)));
      P=PTEMP;
      return(DHGDPT);
}   /* end of function DHGDPT(P,T) */

/**********************************************************************/
float DHGDTP(float P,float T)
/* This function calculates dh/dT |P for light water
   in the superheated gaseous phase.*/
{
      float TS,DHGDTP;
      float TSAT(float P);
      TS=TSAT(P);
      DHGDTP = +(4.5*P/sqrt(7.4529E-06*T*T*T-P*P)
          +0.28*exp(-0.008*(T-162.0))-100.0/T+2.225)
          + (-(4.5*P/pow(7.4529E-06*T*T*T-P*P,1.5))*1.5*T*T*7.4529E-06
          - 0.28*0.008*exp(-0.008*(T-162.0))+100.0/T/T )*(T-TS);
      return(DHGDTP);
}   /* end of function DHGDTP(P,T) */

/**********************************************************************/
float DPDT(float T)
/* This function calculates dP/dT along the saturation line
   for a given temperature.*/
{
      extern char *ERRH2O;
      float DPDT;
      if (T < 17.511)     {
             DPDT=7.406365/270.121*pow((17.511+99.2)/270.121,7.406365-1.0);
             ERRH2O=" DPDT: TEMPERATURE LESS THAN 17.511 deg C    ";
      }
      else if (T > 373.253)     {
             DPDT=4.960785/227.2963*pow((373.253+50.0)/227.2963,4.960785-1.0);
             ERRH2O=" DPDT: TEMPERATURE GREATER THAN 373.253 deg C       ";
      }
      else if(T >= 17.511  &&  T < 56.275)     {
             DPDT=7.406365/270.121*pow((T+99.2)/270.121,7.406365-1.0);
      }
      else if(T >= 56.275  &&  T < 90.88)     {
             DPDT=6.4058216/254.6831*pow((T+78.2)/254.6831,6.4058216-1.0);
      }
      else if (T >= 90.88 && T < 139.781)     {
             DPDT=5.602972/236.2315*pow((T+57.0)/236.2315,5.602972-1.0);
      }
      else if (T > 139.781 && T < 203.662)     {
             DPDT=4.778504/207.9248*pow((T+28.0)/207.9248,4.778504-1.0);
      }
      else if (T >= 203.662 && T <= 299.407)     {
             DPDT=4.304376/185.0779*pow((T+5.0)/185.0779,4.304376-1.0);
      }
      else if (T > 299.407 && T < 355.636)     {
             DPDT=4.460843/195.1819*pow((T+16.0)/195.1819,4.460843-1.0);
      }
      else   {
             DPDT=4.960785/227.2963*pow((T+50.0)/227.2963,4.960785-1.0);
      }
      return(DPDT);
}   /* end of function DPDT(T) */

/**********************************************************************/
float DRFDPT(float P,float T)
/* This function calculates dRHO/dP |T for light water
   in the subcooled liquid phase.*/
{
      float DRFDPT;
      DRFDPT = 170.0/(375.0-T)-0.2;
      return(DRFDPT);
}   /* end of function DRFDPT(P,T) */

/**********************************************************************/
float DRFDTP(float P,float T)
/* This function calculates dRHO/dT |P for light water
   in the subcooled liquid phase.*/
{
     float PS,DRFDTP;
     float PSAT(float T);
     float DPDT(float T);
     float DVFDP(float P);
     float DF(float PS);
     PS=PSAT(T);
     DRFDTP = (170.0/(375.0-T)/(375.0-T))*(P-PS)+DPDT(T)
         * (-DF(PS)*DF(PS)*DVFDP(PS)-170.0/(375.0-T)+0.2);
     return(DRFDTP);
}   /* end of function DRFDTP(P,T) */

/**********************************************************************/
float DRGDPT(float P,float T)
/* This function calculates dRHO/dP |T for light water
   in the superheated gaseous phase.*/
{
     float TS,DVGDPT,DRGDPT;
     float TSAT(float P);
     float DVGDP(float P);
     float DTDP(float P);
     float DGSUP(float P,float T);
     TS=TSAT(P);
     DVGDPT = DVGDP(P)-(0.000466/P-(0.12/(T+100.0)-0.00106)*
              pow(P,0.1)/sqrt(1.96E-08*pow(T+8.,4.0)-P*P))*
              DTDP(P)+(-0.000466/P/P-(0.12/(T+100.0)-0.00106)*0.1*
              pow(P,0.1-1.0)/sqrt(1.96E-08*pow(T+8.,4.0)-P*P)-((0.12/(T+100.0)-
              0.00106)*pow(P,1.1)/pow(1.96E-8*pow(T+8,4.0)-P*P,1.5)))*(T-TS);
     DRGDPT=-DGSUP(P,T)*DGSUP(P,T)*DVGDPT;
     return(DRGDPT);
}  /* end of function DRGDPT(P,T) */

/**********************************************************************/
float DRGDTP(float P,float T)
/* This function calculates dRHO/dT |P for light water
   in the superheated gaseous phase.*/
{
     float TS,DRGDTP,DVGDTP;
     float TSAT(float P);
     float DGSUP(float P,float T);
     TS=TSAT(P);
     DVGDTP =  0.000466/P-( 0.12 / (T+100.0) - 0.00106 )*pow(P,0.1)
              / sqrt ( 1.96E-08 * pow(T+8.0,4.0) -P*P) + (T-TS)*
              ( ( 0.12 / pow(T+100.0,2))*pow(P,0.1)
              / sqrt ( 1.96E-08 * pow(T+8.0,4.0) -P*P) + (0.12 / (T+100.0)
              - 0.00106 )*pow(P,0.1) / pow( 1.96E-08 * pow(T+8.0,4.0) -P*P,1.5)
              /2 * 1.96E-08 * 4.0 * pow(T+8.0,3.0));
     DRGDTP=-DGSUP(P,T)*DGSUP(P,T)*DVGDTP;
     return(DRGDTP);
}   /* end of function DRGDTP(P,T) */

/**********************************************************************/
float DSFDP(float P)
/* This function calculates the derivative with respect to
   pressure of the specific entropy of light water in the
   liquid phase at saturation for a given pressure.*/
{
     extern char *ERRH2O;
     float DSFDP;
     if (P < 0.002)     {
             DSFDP = 4.5397665*0.0829772*pow(0.002,0.0829772-1.0);
             ERRH2O = " DSFDP: PRESSURE LESS THAN 0.002 MPa    ";
     }
     else if (P >= 21.25)     {
             DSFDP = 4.3632383E-05*3.153273*pow(21.25-0.40,3.153273-1.0);
             ERRH2O = " DSFDP: PRESSURE GREATER THAN 21.25 MPa       ";
     }
     else if(P >= 0.002  &&  P < 0.0812)     {
             DSFDP = 4.5397665 * 0.0829772 * pow(P,0.0829772-1.0);
     }
     else if (P >= 0.0812 && P < 1.666)     {
             DSFDP = 3.340244 * 0.125474 * pow(P,0.125474-1.0);
     }
     else if (P >= 1.666 && P < 8.825)     {
             DSFDP = 1.748203 * 0.2275611 * pow(P,0.2275611-1.0);
     }
     else if (P >= 8.825 && P < 16.66)     {
             DSFDP = 0.2549248 * 0.6381866 * pow(P,0.6381866-1.0);
     }
     else         {
             DSFDP = 4.3632383E-05 * 3.153273 * pow(P-0.40,2.153273);
     }
     return(DSFDP);
}   /* end of function DSFDP(P) */

/**********************************************************************/
float DSFDPT(float P,float T)
/* This function calculates d s/d P |T for light water
   in the subcooled liquid phase.*/
{
     extern char *ERRH2O;
     float PS,TT,DSFDPT;
     float PSAT(float T);
     PS=PSAT(T);
     TT=T;
     if (P < PS)     {
            ERRH2O=" DSFDPT: PRESSURE LESS THAN SATURATION  ";
     }
     else if (P > 18.)     {
            ERRH2O=" DSFDPT: PRESSURE GREATER THAN 18 MPa      ";
     }
     else if (T < 17.511)     {
            TT=17.511;
            ERRH2O=" DSFDPT: TEMPERATURE LESS THAN 17.511 deg C   ";
     }
     else if (T > 357.03)     {
            TT=357.03;
            ERRH2O=" DSFDPT: TEMPERATURE GREATER THAN 357.03 deg C      ";
     }
     DSFDPT = 0.0004-0.325/(370.0-TT);
     return(DSFDPT);
}   /* end of function DSFDPT(P,T) */

/**********************************************************************/
float DSFDTP(float P,float T)
/* This function calculates ds/dT |P for light water
   in the subcooled liquid phase.*/
{
     float PS,DSFDTP;
     float PSAT(float T);
     float DPDT(float P);
     float DSFDP(float P);
     PS=PSAT(T);
     DSFDTP = -0.325/pow(370.0-T,2.0) * (P-PS) + DPDT(T)
              * (DSFDP(PS) - 0.0004 + 0.325/(370.0-T));
     return(DSFDTP);
}   /* end of function DSFDTP(P,T) */

/**********************************************************************/
float DSGDP(float P)
/* This function calculates the derivative with respect to
   pressure of the specific entropy of light water in the gas
   phase at saturation for a given pressure.*/
{
     extern char *ERRH2O;
     float DSGDP;
     if (P < 0.002)     {
             DSGDP = 16.644669*(-0.0192733)*pow(0.002,-0.0192733-1.0);
             ERRH2O=" DSGDP: PRESSURE LESS THAN 0.002 MPa    ";
     }
     else if (P > 21.5)     {
             DSGDP = -0.042830642 * 1.779526 * pow(21.5-18.7,0.779526);
             ERRH2O=" DSGDP: PRESSURE GREATER THAN 21.5 MPa       ";
     }
     else if (P >= 0.002  &&  P < 0.0916)     {
             DSGDP = 16.644669*(-0.0192733)* pow(P,-0.0192733-1.0);
     }
     else if (P >= 0.0916 && P <= 1.48)     {
             DSGDP = -0.335924/P;
     }
     else if (P > 1.48 && P <= 8.05)     {
             DSGDP = -1.227644 * 0.2481072 * pow(P,0.2481072-1.0);
     }
     else if (P > 8.05 && P <= 15.64)     {
             DSGDP = -0.084638514 * 0.9082161 * pow(P,0.9082161-1.0);
     }
     else if (P > 15.64 && P <= 20.00)     {
             DSGDP = -3.6897161E-03 * 2.012466 * pow(P-7.80,1.012466);
     }
     else         {
             DSGDP = -0.042830642 * 1.779526 * pow(P-18.7,0.779526);
     }
     return(DSGDP);
}   /* end of function DSGDP(P) */

/**********************************************************************/
float DSGDPT(float P,float T)
/* This function calculates ds/dP |T for light water
   in the superheated gaseous phase.*/
{
     float DSGDPT,TS;
     float TSAT(float P);
     float DSGDP(float P);
     float DTDP(float P);
     TS=TSAT(P);
     DSGDPT = DSGDP(P)-(0.004*pow(P,1.2)/sqrt(3.025E-11*pow(T+46.0,5.0)-P*P)
              + 5.0E-05/sqrt(P) - 4.125E-06 * T + 0.0053)*DTDP(P) +
              (T-TS)*(0.004 * 1.2*pow(P,0.2)/sqrt(3.025E-11*pow(T+46.0,5.0)
              -P*P) + 0.004*pow(P,2.1)/pow(3.025E-11*pow(T+46.0,5.0)-P*P,1.5)
              -5.0E-05/2.0/pow(P,1.5));
     return(DSGDPT);
}   /* end of function DSGDPT(P,T) */

/**********************************************************************/
float DSGDTP(float P,float T)
/* This function calculates ds/dT |P for light water
   in the superheated gaseous phase.*/
{
     float DSGDTP,TS;
     float TSAT(float P);
     TS=TSAT(P);
     DSGDTP = 0.004 * pow(P,1.2)/sqrt(3.025E-11*pow(T+46.0,5.0)-P*P)
              + 5.0E-05/sqrt(P) - 4.125E-06*T + 0.0053 +
              (-0.004*pow(P,1.2)*3.025E-11*5.0*pow(T+46.0,4.0)/2.0/
              pow(3.025E-11*pow(T+46.0,5.0)-P*P,1.5) -
              4.125E-06) * (T-TS);
     return(DSGDTP);
}   /* end of function DSGDTP(P,T) */

/**********************************************************************/
float DTDP(float P)
/* This function calculates dT/dP along the saturation line
   for a given pressure.*/
{
     extern char *ERRH2O;
     float DTDP;
     if (P < 0.002)     {
            DTDP=0.1350197 * 270.121* pow(0.002,0.135019 -1.0);
            ERRH2O=" DTDP: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.85)     {
            DTDP= 0.201581 * 227.2963* pow(21.85,0.201581 - 1.0);
            ERRH2O=" DTDP: PRESSURE GREATER THAN 21.85 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.01672)     {
            DTDP=0.135019 * 270.121 * pow(P,0.135019-1.0);
     }
     else if (P >= 0.01672 && P < 0.0725)     {
            DTDP=0.156108 * 254.6831 * pow(P,0.156108-1.0);
     }
     else if (P >= 0.0725 && P <= 0.359)     {
            DTDP=0.1784767 * 236.2315* pow(P,0.1784767 -1.0);
     }
     else if (P > 0.359 && P <= 1.676)     {
            DTDP=0.2092705 * 207.9248* pow(P,0.2092705 -1.0);
     }
     else if (P > 1.676 && P <= 8.511)     {
            DTDP=0.2323217 * 185.0779* pow(P,0.2323217 -1.0);
     }
     else if (P > 8.511 && P <= 17.69)     {
            DTDP=0.2241729 * 195.1819* pow(P,0.2241729 -1.0);
     }
     else       {
            DTDP= 0.201581 * 227.2963*pow(P,0.201581 - 1.0);
     }
     return(DTDP);
}   /* end of function DTDP(P) */

/**********************************************************************/
float DVFDP(float P)
/* This function calculates the derivative with respect to
   pressure of the specific volume of light water in the liquid
   phase at saturation for a given pressure.*/
{
     extern char *ERRH2O;
     float DVFDP;
     if (P < 0.002)     {
             DVFDP=1.9118E-4*0.546472* pow(0.002,0.546472-1.0);
             ERRH2O=" DVFDP: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.5)     {
             DVFDP= 4.1035988E-24* 15.03329* pow(21.5,14.03329);
             ERRH2O=" DVFDP: PRESSURE GREATER THAN 21.5 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.01468)     {
             DVFDP=1.9118E-4 * 0.546472 * pow(P,0.546472-1.0);
     }
     else if (P >= 0.01468  &&  P < 0.275)     {
             DVFDP=1.380934E-4 * 0.388715 * pow(P,0.388715-1.0);
     }
     else if (P >= 0.275 && P <= 1.0)     {
             DVFDP=1.2746977E-4*0.4644339* pow(P,0.4644339-1.0);
     }
     else if (P > 1.0 && P <= 3.88)     {
             DVFDP=1.0476071E-4*0.5651090* pow(P,0.5651090 -1.0);
     }
     else if (P > 3.88 && P <= 8.84)     {
             DVFDP= 3.2836717E-5;
     }
     else if (P > 8.84 && P <= 14.463)     {
             DVFDP=3.3551046E-4*0.058403566* exp(0.058403566*P);
     }
     else if (P > 14.463 && P < 18.052)     {
             DVFDP= 3.1014626E-8* 3.284754* pow(P,2.284754);
     }
     else if (P >= 18.052 && P < 20.204)     {
             DVFDP= 1.5490787E-11* 5.7205* pow(P,4.7205);
     }
     else   {
             DVFDP= 4.1035988E-24* 15.03329* pow(P,14.03329);
     }
     return(DVFDP);
}   /* end of function DVFDP(P) */
